
package rankingimagenes;

import java.awt.Color;
import java.awt.Dimension;
import javax.swing.JLabel;
import javax.swing.JFileChooser;
import javax.imageio.ImageIO;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import java.awt.Image;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.io.FileFilter;
import java.util.ArrayList;
import javax.swing.BorderFactory;
import java.io.File;
import java.io.IOException;
import javax.swing.JOptionPane;
import com.rankingimagenes.Filtro;

/**
 * @author Diego Vazquez Blanco
 */
public class NegocioPrincipal {
    
    private PresentacionPrincipal mipresentacion;
    private DatosPrincipal misdatos;
    
    /**
     * Contructor de la clase
     * @param vista Referencia del objeto de la vista principal para que el controlador creado tome el control. 
     */
    public NegocioPrincipal(PresentacionPrincipal vista){
        this.mipresentacion = vista;
        this.misdatos = new DatosPrincipal();
        cargarComboBoxFiltros();
    }
    
    public void cargarComboBoxFiltros(){
        ArrayList<String> array_filtros = new ArrayList<>();
        File Filtros = new File("./Filtros");
        
        String[] filtros = Filtros.list();
        
        if (filtros.length > 0){
            // carga las bibliotecas de filtros en un array auxiliar para ordenarlos
            for (int x = 0; x < filtros.length; x++){
                array_filtros.add(filtros[x]);
            }
            array_filtros.sort(null);
            // carga las bibliotecas de filtros en el ComboBox_Filtros
            for (int x = 0; x < filtros.length; x++){
                mipresentacion.getComboBoxFiltros().addItem(array_filtros.get(x));
            }
        }
    }
    
    public void cambiarFiltro(){
        JOptionPane.showMessageDialog(mipresentacion, "Se cerrará la aplicación. "
                                    + "Si la aplicación se ha ejecutado mediante\n"
                                    + "el \"Launcher\", se volverá a iniciar con el "
                                    + "filtro seleccionado.");
        System.exit(mipresentacion.getComboBoxFiltros().getSelectedIndex()+1);
    }
    
    public void cargarImagenesCarpeta(){
        int returnVal;
        int n_imagenes;
        
        // volvemos invisible el scrollpane para que se vea el label con la imagen de cargando
            mipresentacion.getScrollPanePrincipal().setVisible(false);
        
        // Seleccion del directorio y guardado del path en el modelo
        returnVal = seleccionarDirectorio();
        
        // Si no se selecciona directorio (si se cancela), no se ejecutaran los siguientes metodos
        if(returnVal == JFileChooser.APPROVE_OPTION) {

            // Carga de las imagenes del directorio seleccionado en la lista de imagenes del modelo
            cargarListaImagenes();
            
            // carga las imagenes en la lista de labels del modelo
            n_imagenes = cargarLabels();
            
            JOptionPane.showMessageDialog(mipresentacion, "Se han cargado " + n_imagenes + " imagenes. "
                                                            + "A continuación se mostrarán en el panel.");
            
            // Volvemos a hacer visible el scrollpane antes de cargar las imagenes en el panel 
            // para que no se bloque el programa en buble infinito
            mipresentacion.getScrollPanePrincipal().setVisible(true);
            mipresentacion.getComponent(0).validate();
            
            // coloca las imagenes en el panel principal
            mostrarImagenesPrincipal();
        }
        
        // Esto es por si acaso se cancela la seleccion de directorio
            mipresentacion.getScrollPanePrincipal().setVisible(true);
            mipresentacion.getComponent(0).validate();
    }
    
    public int seleccionarDirectorio(){
        JFileChooser chooser = new JFileChooser();
        chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
        int returnVal = chooser.showOpenDialog(mipresentacion);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if (chooser.getSelectedFile().isDirectory()){
                misdatos.setPathCarpetaImagenes(chooser.getSelectedFile().toString());
                mipresentacion.setTextLabelDirectorio(misdatos.getPathCarpetaImagenes());
            }
            else{
                returnVal = JFileChooser.CANCEL_OPTION;
                JOptionPane.showMessageDialog(chooser, "Ha elegido para abrir un Fichero/Archivo. "
                                                        + "Debe elegir Abrir un Directorio/Carpeta.");
                mipresentacion.setTextLabelDirectorio("Presione [Seleccionar Carpeta] para elegir "
                                                        + "el directorio de las imágenes.");
            }
            return returnVal;
        }
        else {
            return returnVal;
        }
    }
    
    public void cargarListaImagenes(){
        
        File f = new File(misdatos.getPathCarpetaImagenes());

        misdatos.getListaImagenesRGB().clear();
        misdatos.getListaMetadatosPrincipal().clear();

        // Recopila las imagenes del directorio descartando los demas tipos de archivo
        File[] ficheros = f.listFiles(
        new FileFilter() {
            @Override
            public boolean accept(File f) {
                String extension = Utils.getExtension(f);
                if (extension != null) {
                    if (extension.equals("jpeg") ||
                        extension.equals("png") ||
                        extension.equals("gif") ||
                        extension.equals("bmp") ||
                        extension.equals("jpg")) {
                            return true;
                    } else {
                        return false;
                    }
                }
                return false;
            }
        }
        );
        
        if (ficheros.length > 0){
            
            // carga imagenes en un array BufferedImage para poder tratar los valores RGB y obtiene los metadatos
            for (int x = 0; x < ficheros.length; x++){
                try{
                    misdatos.addListaImagenesRGB(ImageIO.read(ficheros[x]));
                    misdatos.addListaMetadatosPrincipal(
                            "<html>" + Utils.getMetadata(ficheros[x])
                            + "Height: " + misdatos.getImagenListaRGB(x).getHeight() + "px<br>"
                            + "Width: " + misdatos.getImagenListaRGB(x).getWidth() + "px<br>" 
                            + "Type: " + Utils.getExtension(ficheros[x]).toUpperCase() + "<br>"
                            + "Location: " + ficheros[x].getParent() + "<br>"
                            + "</html>");
                }catch(IOException e){
                    System.err.println("Alguna imagen no se pudo leer." + "\n\n" + "Exception message: " + e);
                }
            }
        }
        else {
            JOptionPane.showMessageDialog(mipresentacion, "Ha seleccionado un directorio que no contiene imágenes "
                                                            + "admitidas (JPEG, PNG, GIF, BMP, JPG)");
            mipresentacion.setTextLabelDirectorio(misdatos.getPathCarpetaImagenes() + " [Sin Imágenes]");
        }
    }

    public int cargarLabels(){
        int ancho;      // pixeles de anchura de la imagen reducida
        int alto;       // pixeles de altura de la imagen reducida
        JLabel jlabel;
        MouseListener mouseListener;
        misdatos.getListaLabelsPrincipal().clear();
        for (int x = 0; x < misdatos.getListaImagenesRGB().size(); x++){
            // Calcula el alto y ancho de la imagen reducida de forma que mantenga la relacion de aspecto
            if (misdatos.getImagenListaRGB(x).getHeight() > misdatos.getImagenListaRGB(x).getWidth()){
                alto = 200;
                ancho = misdatos.getImagenListaRGB(x).getWidth() * alto / misdatos.getImagenListaRGB(x).getHeight();
            }
            else {
                ancho = 200;
                alto = misdatos.getImagenListaRGB(x).getHeight() * ancho / misdatos.getImagenListaRGB(x).getWidth();
            }
            // Prepara la imagen que va a ser insertada en un label
            Icon icono;
            icono = new ImageIcon(misdatos.getImagenListaRGB(x).getScaledInstance(ancho, alto, Image.SCALE_DEFAULT));
            // Creacion de un label, configuracion y añadido al array de labels
            jlabel = new JLabel(icono);
            jlabel.setSize(200, 200);
            jlabel.setBorder(BorderFactory.createLineBorder(Color.gray));
            jlabel.setFocusable(false);
            jlabel.setToolTipText(misdatos.getMetadatosListaPrincipal(x)); // Incluye metadatos de la imagen al label
            mouseListener = new MouseListener() {
                @Override
                public void mouseClicked(MouseEvent e) {
                    JLabel label = (JLabel) e.getComponent();
                    if (label.isFocusable()){
                        label.setFocusable(false);
                        label.setBorder(BorderFactory.createLineBorder(Color.gray));
                    }
                    else {
                        label.setFocusable(true);
                        label.setBorder(BorderFactory.createLineBorder(Color.blue, 5));
                    }
                }
                @Override
                public void mousePressed(MouseEvent e) {
                }
                @Override
                public void mouseReleased(MouseEvent e) {
                }
                @Override
                public void mouseEntered(MouseEvent e) {
                }
                @Override
                public void mouseExited(MouseEvent e) {
                }
            };
            jlabel.addMouseListener(mouseListener);
            misdatos.getListaLabelsPrincipal().add(jlabel);
        }
        return misdatos.getListaLabelsPrincipal().size();
    }
        
    public void mostrarImagenesPrincipal(){
        int fila = 0;   // contador de filas al ir añadiendo imagenes al panel
        int cont = 0;   // lleva el contador para no pasar del numero de columnas que caben
        
        mipresentacion.getPanelPrincipal().removeAll();

        // Coloca cada imagen donde debe (Contenedor cada imagen: 200x200, Margen: 10)
        for (int x = 0; x < misdatos.getListaLabelsPrincipal().size(); x++){

            if (mipresentacion.getScrollPanePrincipal().getWidth()-20 > ((cont+1)*210 + 10)){
                misdatos.getLabelListaPrincipal(x).setLocation(10 + 210*cont, 10 + 210*fila);
                mipresentacion.getPanelPrincipal().add(misdatos.getLabelListaPrincipal(x));
                mipresentacion.getPanelPrincipal().repaint();
                cont++;
            }
            else{
                fila++;
                cont=0;
                x--; // vuelve a la imagen que no pudo poner porque se salía del panel
            }
        }
        
        // Dimensiona el Panel Principal para contener todas las imagenes del directorio
        // quitamos 23px para que no aparezca la scrollbar horizontal
        Dimension d = new Dimension(mipresentacion.getScrollPanePrincipal().getWidth() - 23, 
                                    10 + 210 * (fila+1)/* le sumamos 1 para que cuente la 
                                                            ultima fila para el scroll */);
        mipresentacion.getPanelPrincipal().setPreferredSize(d);
        mipresentacion.getPanelPrincipal().setSize(d);
        mipresentacion.getPanelPrincipal().repaint();
    }
    
    public void aplicarFiltro(){
        int ancho;      // pixeles de anchura de la imagen reducida
        int alto;       // pixeles de altura de la imagen reducida
        ArrayList<BufferedImage> array_seleccionadas_rgb = formarArraySeleccionadas().get(0);
        ArrayList<JLabel> array_seleccionadas_labels = formarArraySeleccionadas().get(1);
        boolean mensaje = false;
        
        mipresentacion.getScrollPanePrincipal().setVisible(false);
        
        for (int x = 0; x < array_seleccionadas_labels.size(); x++){

            // Calcula el alto y ancho de la imagen reducida de forma que mantenga la relacion de aspecto
            if (array_seleccionadas_rgb.get(x).getHeight() > array_seleccionadas_rgb.get(x).getWidth()){
                alto = 200;
                ancho = array_seleccionadas_rgb.get(x).getWidth() * alto / array_seleccionadas_rgb.get(x).getHeight();
            }
            else {
                ancho = 200;
                alto = array_seleccionadas_rgb.get(x).getHeight() * ancho / array_seleccionadas_rgb.get(x).getWidth();
            }
            
            // Prepara la imagen que va a ser insertada en un label
            Icon icono;
            if ((array_seleccionadas_rgb.get(x).getType() == 6 || array_seleccionadas_rgb.get(x).getType() == 13) 
                    && "Escala de Grises".equals(Filtro.nombre_filtro)){
                icono = new ImageIcon(array_seleccionadas_rgb.get(x).getScaledInstance(ancho, alto, Image.SCALE_DEFAULT));
                mensaje = true;
            }
            else if ((array_seleccionadas_rgb.get(x).getType() == 6 || array_seleccionadas_rgb.get(x).getType() == 13) 
                    && "Umbral de 32".equals(Filtro.nombre_filtro)){
                icono = new ImageIcon(array_seleccionadas_rgb.get(x).getScaledInstance(ancho, alto, Image.SCALE_DEFAULT));
                mensaje = true;
            }
            else{
                icono = new ImageIcon(Filtro.aplicar(array_seleccionadas_rgb.get(x)).getScaledInstance(ancho, alto, Image.SCALE_DEFAULT));
            }
            
            // Creacion de un label, configuracion y añadido al array de labels
            array_seleccionadas_labels.get(x).setIcon(icono);
        }
        
        mipresentacion.getScrollPanePrincipal().setVisible(true);
        mipresentacion.getComponent(0).validate();
        
        if (mensaje)
                JOptionPane.showMessageDialog(mipresentacion, "Ha seleccionado para aplicar un filtro a un tipo de imagen "
                        + "a la que no es posible aplicarlo (Ej: PNG y GIF para la Escala de Grises).\n\nA las imágenes seleccionadas "
                        + "de diferente tipo al descrito se les habrá aplicado el filtro correctamente.");
    }
    
    public void seleccionarTodas(){
        for (int x = 0; x < misdatos.getListaLabelsPrincipal().size(); x++){
            misdatos.getLabelListaPrincipal(x).setFocusable(true);
            misdatos.getLabelListaPrincipal(x).setBorder(BorderFactory.createLineBorder(Color.blue, 5));
        }
    }
    
    public void deseleccionarTodas() {
        for (int x = 0; x < misdatos.getListaLabelsPrincipal().size(); x++){
            misdatos.getLabelListaPrincipal(x).setFocusable(false);
            misdatos.getLabelListaPrincipal(x).setBorder(BorderFactory.createLineBorder(Color.gray));
        }
    }
    
    /**
     * Envia las imagenes del panel principal seleccionadas
     */
    public void comparar(){
                
        ArrayList<ArrayList> array_seleccionadas;
        array_seleccionadas = formarArraySeleccionadas();
        
        // Si no se seleccionan imágenes no puede comparar
        if (array_seleccionadas.get(0).isEmpty())
            JOptionPane.showMessageDialog(mipresentacion, "No se ha seleccionado ninguna imagen para comparar.");
        else{
            Main.mostrarRanking(array_seleccionadas, mipresentacion.getComboBoxAlgoritmos().getSelectedItem().toString());
        }
    }
    
    public ArrayList<ArrayList> formarArraySeleccionadas() {
        
        ArrayList<BufferedImage> array_seleccionadas_rgb = new ArrayList<>();
        ArrayList<JLabel> array_seleccionadas_labels = new ArrayList<>();
        ArrayList<String> array_seleccionadas_metadatos = new ArrayList<>();
        ArrayList<File> array_seleccionadas_ficheros = new ArrayList<>();
        ArrayList<ArrayList> array_seleccionadas = new ArrayList<>();
        
        File f = new File(misdatos.getPathCarpetaImagenes());

        // Recopila las imagenes del directorio descartando los demas tipos de archivo
        File[] ficheros = f.listFiles(
        new FileFilter() {
            @Override
            public boolean accept(File f) {
                String extension = Utils.getExtension(f);
                if (extension != null) {
                    if (extension.equals("jpeg") ||
                        extension.equals("png") ||
                        extension.equals("gif") ||
                        extension.equals("bmp") ||
                        extension.equals("jpg")) {
                            return true;
                    } else {
                        return false;
                    }
                }
                return false;
            }
        }
        );
        
        for (int x = 0; x < misdatos.getListaLabelsPrincipal().size(); x++){
            if (misdatos.getLabelListaPrincipal(x).isFocusable()){
                array_seleccionadas_rgb.add(misdatos.getImagenListaRGB(x));
                array_seleccionadas_labels.add(misdatos.getLabelListaPrincipal(x));
                array_seleccionadas_metadatos.add(misdatos.getMetadatosListaPrincipal(x));
                array_seleccionadas_ficheros.add(ficheros[x]);
            }
        }
        
        array_seleccionadas.add(array_seleccionadas_rgb);
        array_seleccionadas.add(array_seleccionadas_labels);
        array_seleccionadas.add(array_seleccionadas_metadatos);
        array_seleccionadas.add(array_seleccionadas_ficheros);
        
        return array_seleccionadas;
    }
    
    public void showInfo(){
        JOptionPane.showMessageDialog(mipresentacion, "Para aplicar un filtro, primero selecciona las imágenes a las que se\nlo quieres aplicar y luego haz "
                                            + "click en el botón [Aplicar Filtro].\n\n"
                                            + "Para cambiar el algoritmo del filtro, seleccionar el filtro deseado y\nhacer click en [Cambiar Filtro].\n\n"
                                            + "Para cambiar el algoritmo de cálculo del ranking, tan sólo hay que\ndejar seleccionado el deseado "
                                            + "antes de hacer click en [Comparar].");
    }
}